"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.copyElementShadowPrototype = exports.ElementShadow = void 0;
const lodash_clonedeep_1 = __importDefault(require("lodash.clonedeep"));
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const logger_1 = require("./../libs/logger");
const variable_1 = require("./../libs/variable");
const root_scene_1 = require("./root-scene");
const IgnoreVarsBeforeExec = ['vars', 'setContextVarsString', 'setContextVarsObject', 'ignoreVarsBeforeExec', 'logger', 'parent', 'if', 'runs', 'elementBuilder', 'globalVars', 'scene'];
const EVAL_EXPS_KEY = Symbol('EvalExps');
class ElementShadow {
    title;
    ignore;
    ignoreError;
    error;
    log;
    vars;
    async;
    loop;
    loopKey;
    loopValue;
    if;
    tag;
    parentData;
    parent;
    setContextVarsString;
    setContextVarsObject;
    execResult;
    ignoreVarsBeforeExec;
    logger;
    scene;
    markElementNeedReplace() {
        const props = Object.keys(this);
        let ignoreVarsBeforeExec = [...IgnoreVarsBeforeExec];
        if (this.ignoreVarsBeforeExec) {
            ignoreVarsBeforeExec.push(...this.ignoreVarsBeforeExec);
        }
        this[EVAL_EXPS_KEY] = props.filter(key => {
            return !ignoreVarsBeforeExec.includes(key) && (0, variable_1.isGetEvalExp)(this[key]);
        });
    }
    async setVarsAfterExec() {
        if (this['vars']) {
            const keys = await this.scene.setVars(this.vars, this.setContextVarsString, this.setContextVarsObject || this);
            this.logger.trace(`[vars] \t%j`, keys.reduce((sum, e) => {
                sum[e] = this.scene.globalVars[e];
                return sum;
            }, {}));
        }
    }
    async evalPropsBeforeExec() {
        const evalExps = this[EVAL_EXPS_KEY];
        await Promise.all(evalExps.map(async (key) => {
            this[key] = await this.scene.getVars(this[key], this);
        }));
    }
    async $$prepare() {
        if (this.logger.is(logger_1.LoggerLevel.INFO) && this.parent?.title) {
            this.logger.addIndent();
        }
        if (this.if === undefined)
            return true;
        this.if = await this.scene.getVars(this.if, this);
        return !!this.if;
    }
    getParentByClassName(ClazzType) {
        let parent = this.parent;
        do {
            if (parent instanceof ClazzType)
                return parent;
        } while (parent = parent['parent']);
        return parent;
    }
    $$extend(ids) {
        if (!ids?.length)
            return;
        if (typeof ids === 'string')
            ids = [ids];
        const props = {};
        ids.forEach(id => {
            const cached = root_scene_1.RootScene.templatesManager.getFromCached(id);
            cached && (0, lodash_merge_1.default)(props, cached);
        });
        return props;
    }
    $$clone(parent) {
        const newElement = (0, lodash_clonedeep_1.default)(this);
        newElement.parent = parent;
        return newElement;
    }
    $$export(id, props) {
        if (!id)
            return;
        root_scene_1.RootScene.templatesManager.pushToCached(id, props);
        this.logger.trace(`->    \t%s`, id);
    }
    init(_props) { }
    dispose() { }
}
exports.ElementShadow = ElementShadow;
const ownerProperties = Object.getOwnPropertyNames(ElementShadow.prototype).filter(k => !['constructor', 'init', 'exec', 'dispose'].includes(k));
function copyElementShadowPrototype(elem) {
    ownerProperties.forEach(k => {
        if (!elem[k])
            elem[k] = ElementShadow.prototype[k];
    });
}
exports.copyElementShadowPrototype = copyElementShadowPrototype;
//# sourceMappingURL=element-shadow.js.map