"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RootScene = void 0;
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const scene_1 = require("./../components/scene/scene");
const events_manager_1 = require("./../managers/events-manager");
const tags_manager_1 = require("./../managers/tags-manager");
const templates_manager_1 = require("./../managers/templates-manager");
const utility_function_manager_1 = require("./../managers/utility-function-manager");
class RootScene extends scene_1.Scene {
    logger;
    static tagsManager;
    static templatesManager;
    static globalUtils = new utility_function_manager_1.UtilityFunctionManager();
    static globalVars;
    static rootDir = '';
    constructor(logger) {
        super(true);
        this.logger = logger;
        this.handleShutdown();
        RootScene.tagsManager = new tags_manager_1.TagsManager(this);
        RootScene.templatesManager = new templates_manager_1.TemplatesManager();
        RootScene.globalVars = {};
    }
    async exec() {
        events_manager_1.globalEvent.emit('scene/exec:before');
        try {
            const rs = await super.exec();
            return rs;
        }
        finally {
            events_manager_1.globalEvent.emit('scene/exec:end');
        }
    }
    async dispose() {
        events_manager_1.globalEvent.emit('scene/dispose:before');
        try {
            await super.dispose();
        }
        finally {
            events_manager_1.globalEvent.emit('scene/dispose:end');
        }
    }
    async init(props) {
        events_manager_1.globalEvent.emit('scene/init:before');
        try {
            await super.init(props);
        }
        finally {
            events_manager_1.globalEvent.emit('scene/init:end');
        }
    }
    handleShutdown() {
        new Array('SIGINT', 'SIGTERM', 'SIGQUIT')
            .forEach(signal => process.on(signal, async () => {
            await this.dispose();
            process.exit(0);
        }));
    }
    static mergeGlobalVars(obj) {
        (0, lodash_merge_1.default)(this.globalVars, obj);
    }
}
exports.RootScene = RootScene;
//# sourceMappingURL=root-scene.js.map